@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2011-2014 Aurimas Cernius
+ * Copyright (C) 2011-2016 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -67,65 +67,19 @@
 namespace gnote {
 
   ActionManager::ActionManager()
-    : m_main_window_actions(Gtk::ActionGroup::create("MainWindow"))
   {
-    populate_action_groups();
     make_app_actions();
     make_app_menu_items();
+
+    register_main_window_action("close-window", NULL, false);
+    register_main_window_action("delete-note", NULL, false);
+    register_main_window_action("important-note", &Glib::Variant<bool>::variant_type(), false);
+    register_main_window_action("enable-spell-check", &Glib::Variant<bool>::variant_type());
+    register_main_window_action("new-notebook", NULL, false);
+    register_main_window_action("move-to-notebook", &Glib::Variant<Glib::ustring>::variant_type(), false);
   }
 
 
-  void ActionManager::populate_action_groups()
-  {
-    Glib::RefPtr<Gtk::Action> action;
-
-    action = Gtk::Action::create(
-      "QuitGNoteAction", Gtk::Stock::QUIT,
-      _("_Quit"), _("Quit Gnote"));
-    m_main_window_actions->add(action, Gtk::AccelKey("<Control>Q"));
-
-    action = Gtk::Action::create(
-      "ShowPreferencesAction", Gtk::Stock::PREFERENCES,
-      _("_Preferences"), _("Gnote Preferences"));
-    m_main_window_actions->add(action);
-
-    action = Gtk::Action::create("ShowHelpAction", Gtk::Stock::HELP,
-      _("_Contents"), _("Gnote Help"));
-    m_main_window_actions->add(action, Gtk::AccelKey("F1"));
-
-    action = Gtk::Action::create(
-      "ShowAboutAction", Gtk::Stock::ABOUT,
-      _("_About"), _("About Gnote"));
-    m_main_window_actions->add(action);
-
-    action = Gtk::Action::create(
-      "TrayIconMenuAction",  _("TrayIcon"));
-    m_main_window_actions->add(action);
-
-    action = Gtk::Action::create(
-      "TrayNewNoteAction", Gtk::Stock::NEW,
-      _("Create _New Note"), _("Create a new note"));
-    m_main_window_actions->add(action);
-
-    action = Gtk::Action::create(
-      "ShowSearchAllNotesAction", Gtk::Stock::FIND,
-      _("_Search All Notes"),  _("Open the Search All Notes window"));
-    m_main_window_actions->add(action);
-  }
-
-  Glib::RefPtr<Gtk::Action> ActionManager::find_action_by_name(const std::string & n) const
-  {
-    Glib::ListHandle<Glib::RefPtr<Gtk::Action> > actions = m_main_window_actions->get_actions();
-    for(Glib::ListHandle<Glib::RefPtr<Gtk::Action> >::const_iterator iter2(actions.begin()); 
-        iter2 != actions.end(); ++iter2) {
-      if((*iter2)->get_name() == n) {
-        return *iter2;
-      }
-    }
-    DBG_OUT("%s not found", n.c_str());
-    return Glib::RefPtr<Gtk::Action>();      
-  }
-
   void ActionManager::make_app_actions()
   {
     add_app_action("new-note");
@@ -216,75 +170,53 @@ namespace gnote {
     return section;
   }
 
-  void ActionManager::add_main_window_search_action(const Glib::RefPtr<Gtk::Action> & action, int order)
+  void ActionManager::register_main_window_action(const Glib::ustring & action, const Glib::VariantType *state_type, bool modifying)
   {
-    add_main_window_action(m_main_window_search_actions, action, order);
-    signal_main_window_search_actions_changed();
-  }
-
-  void ActionManager::remove_main_window_search_action(const std::string & name)
-  {
-    remove_main_window_action(m_main_window_search_actions, name);
-    signal_main_window_search_actions_changed();
-  }
-
-  std::vector<Glib::RefPtr<Gtk::Action> > ActionManager::get_main_window_search_actions()
-  {
-    return get_main_window_actions(m_main_window_search_actions);
-  }
-
-  void ActionManager::add_main_window_action(std::map<int, Glib::RefPtr<Gtk::Action> > & actions,
-                                             const Glib::RefPtr<Gtk::Action> & action, int order)
-  {
-    std::map<int, Glib::RefPtr<Gtk::Action> >::iterator iter = actions.find(order);
-    while(iter != actions.end()) {
-      iter = actions.find(++order);
+    if(m_main_window_actions.find(action) == m_main_window_actions.end()) {
+      m_main_window_actions[action] = state_type;
+      if(!modifying) {
+        m_non_modifying_actions.push_back(action);
+      }
     }
-    actions[order] = action;
-  }
-
-  void ActionManager::remove_main_window_action(std::map<int, Glib::RefPtr<Gtk::Action> > & actions,
-    const std::string & name)
-  {
-    for(std::map<int, Glib::RefPtr<Gtk::Action> >::iterator iter = actions.begin();
-        iter != actions.end(); ++iter) {
-      if(iter->second->get_name() == name) {
-        actions.erase(iter);
-        break;
+    else {
+      if(m_main_window_actions[action] != state_type) {
+        ERR_OUT("Action %s already registerred with different state type", action.c_str());
       }
     }
   }
 
-  std::vector<Glib::RefPtr<Gtk::Action> > ActionManager::get_main_window_actions(
-    std::map<int, Glib::RefPtr<Gtk::Action> > & actions)
+  std::map<Glib::ustring, const Glib::VariantType*> ActionManager::get_main_window_actions() const
   {
-    std::vector<Glib::RefPtr<Gtk::Action> > res;
-    for(std::map<int, Glib::RefPtr<Gtk::Action> >::iterator iter = actions.begin();
-        iter != actions.end(); ++iter) {
-      res.push_back(iter->second);
+    return m_main_window_actions;
+  }
+
+  bool ActionManager::is_modifying_main_window_action(const Glib::ustring & action) const
+  {
+    return std::find(m_non_modifying_actions.begin(), m_non_modifying_actions.end(), action) == m_non_modifying_actions.end();
+  }
+
+  void ActionManager::register_main_window_search_callback(const std::string & id, const Glib::ustring & action,
+                                                    sigc::slot<void, const Glib::VariantBase&> callback)
+  {
+    DBG_ASSERT(m_main_window_search_actions.find(id) == m_main_window_search_actions.end(), "Duplicate callback for main window search");
+
+    m_main_window_search_actions[id] = std::make_pair(action, callback);
+    signal_main_window_search_actions_changed();
+  }
+
+  void ActionManager::unregister_main_window_search_callback(const std::string & id)
+  {
+    m_main_window_search_actions.erase(id);
+    signal_main_window_search_actions_changed();
+  }
+
+  std::map<Glib::ustring, sigc::slot<void, const Glib::VariantBase&>> ActionManager::get_main_window_search_callbacks()
+  {
+    std::map<Glib::ustring, sigc::slot<void, const Glib::VariantBase&>> cbacks;
+    for(auto & iter : m_main_window_search_actions) {
+      cbacks.insert(iter.second);
     }
-    return res;
-  }
-
-  void ActionManager::add_tray_menu_item(Gtk::MenuItem & item)
-  {
-    m_tray_menu_items.push_back(&item);
-  }
-
-  void ActionManager::remove_tray_menu_item(Gtk::MenuItem & item)
-  {
-    for(std::vector<Gtk::MenuItem*>::iterator iter = m_tray_menu_items.begin();
-        iter != m_tray_menu_items.end(); ++iter) {
-      if(*iter == &item) {
-        m_tray_menu_items.erase(iter);
-        break;
-      }
-    }
-  }
-
-  std::vector<Gtk::MenuItem*> ActionManager::get_tray_menu_items()
-  {
-    return m_tray_menu_items;
+    return cbacks;
   }
 
 }
