@@ -32,6 +32,7 @@
 #include <gtkmm/separatormenuitem.h>
 
 #include "debug.hpp"
+#include "addinmanager.hpp"
 #include "iconmanager.hpp"
 #include "mainwindow.hpp"
 #include "note.hpp"
@@ -45,6 +46,7 @@
 #include "itagmanager.hpp"
 #include "notebooks/notebookmanager.hpp"
 #include "sharp/exception.hpp"
+#include "mainwindowaction.hpp"
 
 
 namespace gnote {
@@ -176,6 +178,20 @@ namespace gnote {
     if(parent) {
       parent->set_focus(*m_editor);
     }
+
+    // Don't allow deleting the "Start Here" note...
+    if(!m_note.is_special()) {
+      m_delete_note_slot = current_host->find_action("delete-note")->signal_activate()
+        .connect(sigc::mem_fun(*this, &NoteWindow::on_delete_button_clicked));
+    }
+
+    MainWindowAction::Ptr important_action = current_host->find_action("important-note");
+    important_action->set_state(Glib::Variant<bool>::create(m_note.is_pinned()));
+    m_important_note_slot = important_action->signal_change_state()
+      .connect(sigc::mem_fun(*this, &NoteWindow::on_pin_button_clicked));
+    notebooks::NotebookManager::obj().signal_note_pin_status_changed
+      .connect(sigc::mem_fun(*this, &NoteWindow::on_pin_status_changed));
+
   }
 
   void NoteWindow::background()
@@ -202,6 +218,8 @@ namespace gnote {
     }
 
     m_note.save();  // to update not title immediately in notes list
+    m_delete_note_slot.disconnect();
+    m_important_note_slot.disconnect();
   }
 
   void NoteWindow::hint_size(int & width, int & height)
@@ -286,58 +304,43 @@ namespace gnote {
     return m_embeddable_toolbar;
   }
 
-  std::vector<Glib::RefPtr<Gtk::Action> > NoteWindow::get_widget_actions()
+  std::vector<Gtk::Widget*> NoteWindow::get_popover_widgets()
   {
-    std::vector<Glib::RefPtr<Gtk::Action> > res;
-    for(std::map<int, Glib::RefPtr<Gtk::Action> >::iterator iter = m_widget_actions.begin();
-        iter != m_widget_actions.end(); ++iter) {
-      res.push_back(iter->second);
+    std::map<int, Gtk::Widget*> widget_map;
+    NoteManager & manager = static_cast<NoteManager&>(m_note.manager());
+    Note::Ptr note = std::dynamic_pointer_cast<Note>(m_note.shared_from_this());
+    FOREACH(NoteAddin *addin, manager.get_addin_manager().get_note_addins(note)) {
+      utils::merge_ordered_maps(widget_map, addin->get_actions_popover_widgets());
     }
 
-    res.push_back(Glib::RefPtr<Gtk::Action>());
-    res.push_back(m_important_action);
-    if(m_delete_action) {
-      // Separator before delete
-      res.push_back(m_delete_action);
+    std::vector<Gtk::Widget*> widgets;
+    for(std::map<int, Gtk::Widget*>::iterator iter = widget_map.begin();
+        iter != widget_map.end(); ++iter) {
+      widgets.push_back(iter->second);
+    }
+
+    widgets.push_back(utils::create_popover_button("win.important-note", _("Is Important")));
+    widgets.push_back(NULL);
+    widgets.push_back(utils::create_popover_button("win.delete-note", _("_Delete")));
+
+    return widgets;
+  }
+
+  std::vector<MainWindowAction::Ptr> NoteWindow::get_widget_actions()
+  {
+    std::vector<MainWindowAction::Ptr> res;
+    EmbeddableWidgetHost *h = host();
+    if(h != NULL) {
+      h->find_action("important-note");
+      h->find_action("delete-note");
     }
     return res;
   }
 
-  sigc::signal<void> & NoteWindow::signal_actions_changed()
-  {
-    return m_signal_actions_changed;
-  }
-
-  void NoteWindow::add_widget_action(const Glib::RefPtr<Gtk::Action> & action, int order)
-  {
-    std::map<int, Glib::RefPtr<Gtk::Action> >::iterator iter = m_widget_actions.find(order);
-    while(iter != m_widget_actions.end()) {
-      iter = m_widget_actions.find(++order);
-    }
-    m_widget_actions[order] = action;
-    m_signal_actions_changed();
-    if(Glib::RefPtr<NonModifyingNoteAction>::cast_dynamic(action) == 0) {
-      action->set_sensitive(m_enabled);
-    }
-  }
-
-  void NoteWindow::remove_widget_action(const std::string & name)
-  {
-    for(std::map<int, Glib::RefPtr<Gtk::Action> >::iterator iter = m_widget_actions.begin();
-        iter != m_widget_actions.end(); ++iter) {
-      if(iter->second->get_name() == name) {
-        m_widget_actions.erase(iter);
-        break;
-      }
-    }
-    m_signal_actions_changed();
-  }
-
-
     // Delete this Note.
     //
 
-  void NoteWindow::on_delete_button_clicked()
+  void NoteWindow::on_delete_button_clicked(const Glib::VariantBase&)
   {
     // Prompt for note deletion
     std::list<Note::Ptr> single_note_list;
@@ -418,21 +421,6 @@ namespace gnote {
     text_button->set_tooltip_text(_("Set properties of text"));
     m_text_menu->property_attach_widget() = text_button;
 
-    m_important_action = utils::CheckAction::create("mark-important");
-    m_important_action->set_label(_("Is Important"));
-    m_important_action->set_tooltip(_("Toggle notes presence in Important Notes notebook"));
-    m_important_action->checked(m_note.is_pinned());
-    m_important_action->signal_activate().connect(sigc::mem_fun(*this, &NoteWindow::on_pin_button_clicked));
-    notebooks::NotebookManager::obj().signal_note_pin_status_changed
-      .connect(sigc::mem_fun(*this, &NoteWindow::on_pin_status_changed));
-
-      // Don't allow deleting the "Start Here" note...
-    if(!m_note.is_special()) {
-      m_delete_action = Gtk::Action::create("delete-note", _("_Delete"), _("Delete this note"));
-      m_delete_action->signal_activate()
-        .connect(sigc::mem_fun(*this, &NoteWindow::on_delete_button_clicked));
-    }
-
     grid->property_margin_left() = 12;
     grid->show_all();
     return grid;
@@ -596,12 +584,20 @@ namespace gnote {
     if(&m_note != &note) {
       return;
     }
-    m_important_action->checked(pinned);
+    EmbeddableWidgetHost *h = host();
+    if(h != NULL) {
+      h->find_action("important-note")->change_state(Glib::Variant<bool>::create(pinned));
+    }
   }
 
-  void NoteWindow::on_pin_button_clicked()
+  void NoteWindow::on_pin_button_clicked(const Glib::VariantBase & state)
   {
-    m_note.set_pinned(!m_note.is_pinned());
+    EmbeddableWidgetHost *h = host();
+    if(h != NULL) {
+      Glib::Variant<bool> new_state = Glib::VariantBase::cast_dynamic<Glib::Variant<bool> >(state);
+      m_note.set_pinned(new_state.get());
+      h->find_action("important-note")->set_state(state);
+    }
   }
 
   void NoteWindow::on_text_button_clicked()
@@ -617,10 +613,10 @@ namespace gnote {
     embeddable_toolbar()->set_sensitive(m_enabled);
     if(m_global_keys)
       m_global_keys->enabled(m_enabled);
-    FOREACH(const Glib::RefPtr<Gtk::Action> & action, get_widget_actions()) {
+    FOREACH(const MainWindowAction::Ptr & action, get_widget_actions()) {
       // A list includes empty actions to mark separators, non-modifying actions are always enabled
       if(action != 0 && Glib::RefPtr<NonModifyingNoteAction>::cast_dynamic(action) == 0) {
-        action->set_sensitive(enable);
+        action->set_enabled(enable);
       }
     }
   }
