@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2015 Aurimas Cernius
+ * Copyright (C) 2010-2016 Aurimas Cernius
  * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -32,6 +32,7 @@
 #include <gtkmm/stock.h>
 
 #include "debug.hpp"
+#include "iactionmanager.hpp"
 #include "ignote.hpp"
 #include "note.hpp"
 #include "notemanager.hpp"
@@ -39,7 +40,6 @@
 #include "recentchanges.hpp"
 #include "sharp/string.hpp"
 
-
 namespace gnote {
 
   NoteRecentChanges::NoteRecentChanges(NoteManager& m)
@@ -100,7 +100,29 @@ namespace gnote {
     m_keybinder.add_accelerator(sigc::mem_fun(*this, &NoteRecentChanges::close_window),
                                 GDK_KEY_Q, Gdk::CONTROL_MASK, (Gtk::AccelFlags)0);
 
-    m_window_menu_default = make_window_menu(m_window_actions_button, std::vector<Gtk::MenuItem*>());
+    std::map<Glib::ustring, const Glib::VariantType*> actions = IActionManager::obj().get_main_window_actions();
+    for(std::map<Glib::ustring, const Glib::VariantType*>::iterator iter = actions.begin();
+        iter != actions.end(); ++iter) {
+      MainWindowAction::Ptr action;
+      if(iter->second == NULL) {
+        add_action(action = MainWindowAction::create(iter->first));
+      }
+      else if(iter->second == &Glib::Variant<bool>::variant_type()) {
+        add_action(action = MainWindowAction::create(iter->first, false));
+      }
+      else if(iter->second == &Glib::Variant<gint32>::variant_type()) {
+        add_action(action = MainWindowAction::create(iter->first, 0));
+      }
+      else if(iter->second == &Glib::Variant<Glib::ustring>::variant_type()) {
+        add_action(action = MainWindowAction::create(iter->first, Glib::ustring("")));
+      }
+      if(action) {
+        action->is_modifying(IActionManager::obj().is_modifying_main_window_action(iter->first));
+      }
+    }
+    find_action("close-window")->signal_activate()
+      .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_close_window));
+
     embed_widget(m_search_notes_widget);
   }
 
@@ -113,12 +135,6 @@ namespace gnote {
     if(m_entry_changed_timeout) {
       delete m_entry_changed_timeout;
     }
-    if(m_window_menu_embedded) {
-      delete m_window_menu_embedded;
-    }
-    if(m_window_menu_default) {
-      delete m_window_menu_default;
-    }
   }
 
   void NoteRecentChanges::make_header_bar()
@@ -366,6 +382,12 @@ namespace gnote {
   }
 
 
+  void NoteRecentChanges::on_close_window(const Glib::VariantBase&)
+  {
+    close_window();
+  }
+
+
   bool NoteRecentChanges::on_delete(GdkEventAny *)
   {
     close_window();
@@ -513,10 +535,11 @@ namespace gnote {
 
     try {
       HasActions &has_actions = dynamic_cast<HasActions&>(widget);
-      m_current_embedded_actions_slot = has_actions.signal_actions_changed().connect(
-        boost::bind(sigc::mem_fun(*this, &NoteRecentChanges::on_main_window_actions_changed),
-                    &m_window_menu_embedded));
-      on_main_window_actions_changed(&m_window_menu_embedded);
+      if(m_window_menu_embedded) {
+        m_window_menu_embedded = NULL;
+      }
+      m_signal_popover_widgets_changed_cid = has_actions.signal_popover_widgets_changed
+        .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_popover_widgets_changed));
     }
     catch(std::bad_cast&) {
     }
@@ -531,11 +554,10 @@ namespace gnote {
       Gtk::Widget &wid = dynamic_cast<Gtk::Widget&>(widget);
       widget.background();
       m_embed_box.remove(wid);
-
+      m_signal_popover_widgets_changed_cid.disconnect();
       m_current_embedded_name_slot.disconnect();
-      m_current_embedded_actions_slot.disconnect();
+
       if(m_window_menu_embedded) {
-        delete m_window_menu_embedded;
         m_window_menu_embedded = NULL;
       }
     }
@@ -567,6 +589,30 @@ namespace gnote {
     return false;
   }
 
+  void NoteRecentChanges::add_action(const MainWindowAction::Ptr & action)
+  {
+    m_actions[action->get_name()] = action;
+    MainWindow::add_action(action);
+  }
+
+  MainWindowAction::Ptr NoteRecentChanges::find_action(const Glib::ustring & name)
+  {
+    std::map<Glib::ustring, MainWindowAction::Ptr>::iterator iter = m_actions.find(name);
+    if(iter != m_actions.end()) {
+      return iter->second;
+    }
+    return MainWindowAction::Ptr();
+  }
+
+  void NoteRecentChanges::enabled(bool is_enabled)
+  {
+    for(auto & iter : m_actions) {
+      if(iter.second->is_modifying()) {
+        iter.second->set_enabled(is_enabled);
+      }
+    }
+  }
+
   EmbeddableWidget *NoteRecentChanges::currently_embedded()
   {
     std::vector<Gtk::Widget*> children = m_embed_box.get_children();
@@ -678,61 +724,82 @@ namespace gnote {
   {
     HasActions *embed_with_actions = dynamic_cast<HasActions*>(currently_embedded());
     if(embed_with_actions) {
-      utils::popup_menu(*m_window_menu_embedded, NULL);
+      if(m_window_menu_embedded == NULL) {
+        m_window_menu_embedded = make_window_menu(m_window_actions_button, embed_with_actions->get_popover_widgets());
+      }
+      m_window_menu_embedded->show_all();
     }
     else {
-      utils::popup_menu(*m_window_menu_default, NULL);
+      if(m_window_menu_default == NULL) {
+        m_window_menu_default = make_window_menu(m_window_actions_button, std::vector<Gtk::Widget*>());
+      }
+      m_window_menu_default->show_all();
     }
   }
 
-  Gtk::Menu *NoteRecentChanges::make_window_menu(Gtk::Button *button, const std::vector<Gtk::MenuItem*> & items)
+  Gtk::Grid *NoteRecentChanges::create_inner_popover_grid(int & top)
   {
-    Gtk::Menu *menu = new Gtk::Menu;
-    for(std::vector<Gtk::MenuItem*>::const_iterator iter = items.begin(); iter != items.end(); ++iter) {
-      menu->append(**iter);
+    Gtk::Grid *grid = manage(new Gtk::Grid);
+    utils::set_common_popover_widget_props(*grid);
+    top = 0;
+    return grid;
+  }
+
+  Gtk::PopoverMenu *NoteRecentChanges::make_window_menu(Gtk::Button *button, const std::vector<Gtk::Widget*> & items)
+  {
+    std::map<Glib::ustring, Gtk::Widget*> submenus;
+    Gtk::PopoverMenu *menu = manage(new Gtk::PopoverMenu);
+    Gtk::Grid *main_grid = manage(new Gtk::Grid);
+    int main_top = 0;
+    int top = 0;
+    Gtk::Grid *grid = create_inner_popover_grid(top);
+    FOREACH(Gtk::Widget *item, items) {
+      if(item) {
+        utils::PopoverSubmenu *submenu = dynamic_cast<utils::PopoverSubmenu*>(item);
+        if(submenu) {
+          submenus[submenu->name()] = item;
+        }
+        else {
+          grid->attach(*manage(item), 0, top++, 1, 1);
+        }
+      }
+      else {
+        main_grid->attach(*grid, 0, main_top++, 1, 1);
+        grid = create_inner_popover_grid(top);
+      }
     }
-    if(items.size()) {
-      menu->append(*manage(new Gtk::SeparatorMenuItem));
+
+    if(top > 0) {
+      main_grid->attach(*grid, 0, main_top++, 1, 1);
+      grid = create_inner_popover_grid(top);
     }
-    Gtk::MenuItem *item = manage(new Gtk::MenuItem(_("_Close"), true));
-    item->add_accelerator("activate", get_accel_group(), GDK_KEY_W, Gdk::CONTROL_MASK, Gtk::ACCEL_VISIBLE);
-    item->add_accelerator("activate", get_accel_group(), GDK_KEY_Q, Gdk::CONTROL_MASK, Gtk::ACCEL_VISIBLE);
-    item->signal_activate().connect(sigc::mem_fun(*this, &NoteRecentChanges::close_window));
-    menu->append(*item);
-    menu->property_attach_widget() = button;
-    menu->show_all();
+
+    Gtk::Widget *close_item = manage(utils::create_popover_button("win.close-window", _("_Close")));
+    close_item->add_accelerator("activate", get_accel_group(), GDK_KEY_W, Gdk::CONTROL_MASK, Gtk::ACCEL_VISIBLE);
+    close_item->add_accelerator("activate", get_accel_group(), GDK_KEY_Q, Gdk::CONTROL_MASK, Gtk::ACCEL_VISIBLE);
+    grid->attach(*close_item, 0, top++, 1, 1);
+
+    main_grid->attach(*grid, 0, main_top++, 1, 1);
+    menu->add(*main_grid);
+    for(auto & submenu : submenus) {
+      menu->add(*submenu.second);
+      menu->child_property_submenu(*submenu.second) = submenu.first;
+    }
+    menu->set_relative_to(*button);
+    menu->set_modal(true);
+    menu->set_position(Gtk::POS_BOTTOM);
     return menu;
   }
 
-  std::vector<Gtk::MenuItem*> & NoteRecentChanges::make_menu_items(std::vector<Gtk::MenuItem*> & items,
-    const std::vector<Glib::RefPtr<Gtk::Action> > & actions)
-  {
-    FOREACH(Glib::RefPtr<Gtk::Action> action, actions) {
-      Gtk::MenuItem *item = manage(action ? action->create_menu_item() : new Gtk::SeparatorMenuItem);
-      items.push_back(item);
-    }
-    return items;
-  }
-
   void NoteRecentChanges::on_embedded_name_changed(const std::string & name)
   {
     set_title(name);
   }
 
-  void NoteRecentChanges::on_main_window_actions_changed(Gtk::Menu **menu)
+  void NoteRecentChanges::on_popover_widgets_changed()
   {
-    if(*menu) {
-      delete *menu;
-      *menu = NULL;
-    }
-
-    HasActions *embed_with_actions = dynamic_cast<HasActions*>(currently_embedded());
-    if(embed_with_actions) {
-      if(!m_window_menu_embedded) {
-        std::vector<Gtk::MenuItem*> items;
-        m_window_menu_embedded = make_window_menu(m_window_actions_button,
-          make_menu_items(items, embed_with_actions->get_widget_actions()));
-      }
+    if(m_window_menu_embedded) {
+      m_window_menu_embedded = NULL;
     }
   }
 
