@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2015 Aurimas Cernius
+ * Copyright (C) 2010-2016 Aurimas Cernius
  * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -29,8 +29,10 @@
 #include <gtkmm/alignment.h>
 #include <gtkmm/applicationwindow.h>
 #include <gtkmm/grid.h>
+#include <gtkmm/popovermenu.h>
 
 #include "base/macros.hpp"
+#include "mainwindowaction.hpp"
 #include "note.hpp"
 #include "searchnoteswidget.hpp"
 #include "utils.hpp"
@@ -60,6 +62,8 @@ public:
     }
   virtual bool contains(EmbeddableWidget &) override;
   virtual bool is_foreground(EmbeddableWidget &) override;
+  virtual MainWindowAction::Ptr find_action(const Glib::ustring & name) override;
+  virtual void enabled(bool is_enabled) override;
 protected:
   virtual void present_note(const Note::Ptr & note) override;
   virtual void on_show() override;
@@ -83,13 +87,14 @@ private:
   void on_search_button_toggled();
   void on_find_next_button_clicked();
   void on_find_prev_button_clicked();
-  Gtk::Menu *make_window_menu(Gtk::Button *button, const std::vector<Gtk::MenuItem*> & items);
-  std::vector<Gtk::MenuItem*> & make_menu_items(std::vector<Gtk::MenuItem*> & items,
-                                                const std::vector<Glib::RefPtr<Gtk::Action> > & actions);
+  Gtk::PopoverMenu *make_window_menu(Gtk::Button *button, const std::vector<Gtk::Widget*> & items);
+  Gtk::Grid *create_inner_popover_grid(int & top);
   void on_embedded_name_changed(const std::string & name);
-  void on_main_window_actions_changed(Gtk::Menu **menu);
   void on_settings_changed(const Glib::ustring & key);
   bool on_notes_widget_key_press(GdkEventKey*);
+  void on_close_window(const Glib::VariantBase&);
+  void add_action(const MainWindowAction::Ptr & action);
+  void on_popover_widgets_changed();
 
   NoteManager        &m_note_manager;
   Gtk::Widget        *m_header_bar;
@@ -107,13 +112,14 @@ private:
   std::list<EmbeddableWidget*> m_embedded_widgets;
   bool                m_mapped;
   sigc::connection    m_current_embedded_name_slot;
-  sigc::connection    m_current_embedded_actions_slot;
+  sigc::connection    m_signal_popover_widgets_changed_cid;
   utils::InterruptableTimeout *m_entry_changed_timeout;
-  Gtk::Menu          *m_window_menu_embedded;
-  Gtk::Menu          *m_window_menu_default;
+  Gtk::PopoverMenu     *m_window_menu_embedded;
+  Gtk::PopoverMenu     *m_window_menu_default;
   utils::GlobalKeybinder m_keybinder;
   bool                m_open_notes_in_new_window;
   bool                m_close_note_on_escape;
+  std::map<Glib::ustring, MainWindowAction::Ptr> m_actions;
 };
 
 
