@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2014 Aurimas Cernius
+ * Copyright (C) 2010-2015 Aurimas Cernius
  * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -39,6 +39,7 @@
 #include <gtkmm/icontheme.h>
 #include <gtkmm/image.h>
 #include <gtkmm/label.h>
+#include <gtkmm/modelbutton.h>
 #include <gtkmm/stock.h>
 #include <gtkmm/textbuffer.h>
 
@@ -112,6 +113,19 @@ namespace gnote {
         cond->signal();
         mutex->unlock();
       }
+
+
+      class PopoverSubmenuGrid
+        : public Gtk::Grid
+        , public PopoverSubmenu
+      {
+      public:
+        PopoverSubmenuGrid(const Glib::ustring & submenu)
+          : PopoverSubmenu(submenu)
+        {
+          set_common_popover_widget_props(*this);
+        }
+      };
     }
 
 
@@ -273,6 +287,56 @@ namespace gnote {
     }
 
 
+    Gtk::Widget * create_popover_button(const Glib::ustring & action, const Glib::ustring & label)
+    {
+      Gtk::ModelButton *item = new Gtk::ModelButton;
+      gtk_actionable_set_action_name(GTK_ACTIONABLE(item->gobj()), action.c_str());
+      item->set_label(label);
+      item->set_use_underline(true);
+      set_common_popover_widget_props(*item);
+      return item;
+    }
+
+
+    Gtk::Widget * create_popover_submenu_button(const Glib::ustring & submenu, const Glib::ustring & label)
+    {
+      Gtk::ModelButton *button = new Gtk::ModelButton;
+      button->property_menu_name() = submenu;
+      button->set_label(label);
+      button->set_use_underline(true);
+      set_common_popover_widget_props(*button);
+      return button;
+    }
+
+
+    Gtk::Grid * create_popover_submenu(const Glib::ustring & name)
+    {
+      return new PopoverSubmenuGrid(name);
+    }
+
+
+    void set_common_popover_widget_props(Gtk::Widget & widget)
+    {
+      widget.property_margin_top() = 5;
+      widget.property_margin_bottom() = 5;
+    }
+
+
+    void add_item_to_ordered_map(std::map<int, Gtk::Widget*> & dest, int order, Gtk::Widget *item)
+    {
+      for(; dest.find(order) != dest.end(); ++order);
+      dest[order] = item;
+    }
+
+
+    void merge_ordered_maps(std::map<int, Gtk::Widget*> & dest, const std::map<int, Gtk::Widget*> & adds)
+    {
+      for(std::map<int, Gtk::Widget*>::const_iterator iter = adds.begin(); iter != adds.end(); ++iter) {
+        add_item_to_ordered_map(dest, iter->first, iter->second);
+      }
+    }
+
+
     void GlobalKeybinder::add_accelerator(const sigc::slot<void> & handler, guint key, 
                                           Gdk::ModifierType modifiers, Gtk::AccelFlags flags)
     {
