@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2011-2013 Aurimas Cernius
+ * Copyright (C) 2011-2013,2015-2016 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -42,10 +42,6 @@
 #include "sharp/exception.hpp"
 #include "sharp/uri.hpp"
 
-#ifdef HAVE_X11_SUPPORT
-#include "libtomboy/tomboyutil.h"
-#endif
-
 namespace sharp {
   class DateTime;
 }
@@ -67,6 +63,29 @@ namespace gnote {
     void main_context_invoke(const sigc::slot<void> & slot);
     void main_context_call(const sigc::slot<void> & slot);
 
+    Gtk::Widget * create_popover_button(const Glib::ustring & action, const Glib::ustring & label);
+    Gtk::Widget * create_popover_submenu_button(const Glib::ustring & submenu, const Glib::ustring & label);
+    Gtk::Grid * create_popover_submenu(const Glib::ustring & name);
+    void set_common_popover_widget_props(Gtk::Widget & widget);
+
+    void add_item_to_ordered_map(std::map<int, Gtk::Widget*> & dest, int order, Gtk::Widget *item);
+    void merge_ordered_maps(std::map<int, Gtk::Widget*> & dest, const std::map<int, Gtk::Widget*> & adds);
+
+    class PopoverSubmenu
+    {
+    public:
+      PopoverSubmenu(const Glib::ustring & name)
+        : m_name(name)
+      {}
+
+      const Glib::ustring & name() const
+        {
+          return m_name;
+        }
+    private:
+      const Glib::ustring m_name;
+    };
+
     class GlobalKeybinder
     {
     public:
@@ -217,26 +236,6 @@ namespace gnote {
       guint m_timeout_id;
     };
 
-    class ForcedPresentWindow 
-      : public Gtk::ApplicationWindow
-    {
-    public:
-      ForcedPresentWindow(const Glib::ustring & title)
-        : Gtk::ApplicationWindow()
-        {
-          Gtk::ApplicationWindow::set_title(title);
-        }
-
-      void present()
-        {
-#ifdef HAVE_X11_SUPPORT
-          ::tomboy_window_present_hardcore(GTK_WINDOW(gobj()));
-#else
-          Gtk::ApplicationWindow::present();
-#endif
-        }
-    };
-
     class ToolMenuButton
       : public Gtk::ToggleToolButton
     {
